==> relations-0.0.1.tar.gz: lib/models/graph.dart <==
import 'graph_edge.dart';
import 'graph_node.dart';

class Graph {

  List<GraphNode> nodes;
  Map<String, List<GraphEdge>> edges;
  List<GraphEdge> allEdges;

  Graph() {
    this.nodes = List<GraphNode>();
    this.edges = Map<String, List<GraphEdge>>();
    this.allEdges = List<GraphEdge>();
  }

  Graph.fromNodes(List<GraphNode> nodesInitial, {Map<String, List<String>> connections}) {
    //
    //{
    //  "node-1" : ["node-6", "node-3"]
    //  "node-3" : ["node-1"]
    //}
    
    this.nodes = List<GraphNode>.from(nodesInitial);
    this.edges = Map<String, List<GraphEdge>>();
    this.allEdges = List<GraphEdge>();

    if (connections != null) {
      _buildEdges(connections);
    }
  }

  Graph copy() {
    Graph ret = Graph();
    ret.nodes = List<GraphNode>.from(this.nodes);
    ret.edges = Map<String, List<GraphEdge>>.from(this.edges);
    ret.allEdges = List<GraphEdge>.from(this.allEdges);
    return ret;

  }
  
  void _buildEdges(Map<String, List<String>> connections) {
    connections.forEach((k,v) {
      v.forEach((String id) {
        connectNodes(k, id);
      });
    });
  }

  bool nodeExists(String id) {
    return nodes.indexWhere((GraphNode n) => n.id == id) > -1;
  }

  void addNode(GraphNode n) {
    if (nodeExists(n.id)) {
      return;
    }

    nodes.add(n);
  }

  void _deleteNode(String id) {
        // remove connected edges
    if (edges.containsKey(id)) {
      for (GraphEdge e in edges[id]) {
          final int idx = edges[e.to].indexWhere((GraphEdge temp) => temp.to == e.from);
          if(idx < 0) continue;
          edges[e.to].removeAt(idx);

          allEdges.remove(e);
      }

      edges[id].clear();
      edges.remove(id);
    }
    nodes.removeWhere((GraphNode node) => node.id == id);
  }

  void removeNodeByID(String id) {
     _deleteNode(id);
  }

  void removeNode(GraphNode n) {
    _deleteNode(n.id);
  }

  bool checkConnection(String idA, String idB) {
    GraphEdge temp = GraphEdge(GraphEdge.INVALID_ID, idA, idB);
    if (edges[idA] == null) return false;
    
    final idx = edges[idA].indexWhere((GraphEdge e) => e.equalByEndPoints(temp));
    return idx > -1;
  }

  bool connectNodes(String idA, String idB) {
    if (checkConnection(idA, idB)) {
      // print('CONNECTION REFUSED - Nodes have been already connected.');
      return false;
    }
    String newEdgeID =  GraphEdge.nextID;
    _addEdge(GraphEdge(newEdgeID, idA, idB));
    return true;
  }

  void _addEdge(GraphEdge e) {

    allEdges.add(e);

    // create, if not created yet
    edges[e.from] ??= List<GraphEdge>();
    edges[e.to] ??= List<GraphEdge>();
    

    GraphEdge other = GraphEdge(e.id, e.to, e.from);
    edges[e.from].add(e);
    edges[e.to].add(other);
  }

  void disconnectNodes(String from, String to) {
    GraphEdge temp = GraphEdge(GraphEdge.INVALID_ID, from, to);
    
    // should be one element
    edges[from]?.removeWhere((GraphEdge e) {
      return e.to == to;
    });
    edges[to]?.removeWhere((GraphEdge e) {
      return e.to == from;
    });

    allEdges.removeWhere((GraphEdge e) => e.equalByEndPoints(temp) );
  }

  GraphNode getNode(String id) {
    return nodes.firstWhere((GraphNode n) => n.id == id, orElse: () => null);
  }

  List<GraphEdge> getEdges(String id) {
    return edges[id];
  }

  Map<String, List<String>> showConnections() {
    // TODO: implement
    return null;
  }

  @override String toString() {
    String nodesStr = nodes.fold<String>('', (String prev, GraphNode n) => prev + '\n${n.toString()}');
    String edgesStr = '';
    edges.keys.forEach((String key) {
      edgesStr += '$key: to ->';
      edges[key].forEach((GraphEdge e) {
        String connectedNode = key == e.from ? e.to : e.from;
        
        edgesStr += ' [id:$connectedNode, biDir:${e.biDirectional}]';
      });
      edgesStr += '\n';
    });

    return nodesStr + '\n' + edgesStr;
  }
}


/*
if (edges.containsKey(id)) {
     
      for(GraphEdge e in edges[id]) {
        print('in for');
      // if (id == e.to) {
      //    print('\'FROM\' remove');
      //    edges[e.from].remove(e);
      //  }
      //  else
      //  if (id == e.from) {
          final int idx = edges[e.to].indexWhere((GraphEdge temp) => temp.to == e.from);
          if (idx < 0) continue;
          edges[e.to].removeAt(idx);

          allEdges.remove(e);
        //}
      }

      edges[id].clear();
      edges.remove(id);
    }
    nodes.removeWhere((GraphNode node) => node.id == id);
    // nodes = nodes.where((GraphNode node) => node.id != id).toList();
*/

==> relations-0.0.3.tar.gz: lib/models/graph.dart <==
import 'graph_edge.dart';
import 'graph_node.dart';

class Graph {
  List<GraphNode> nodes;
  Map<String, List<GraphEdge>> edges;
  List<GraphEdge> allEdges;

  Graph() {
    this.nodes = List<GraphNode>();
    this.edges = Map<String, List<GraphEdge>>();
    this.allEdges = List<GraphEdge>();
  }

  Graph.fromNodes(List<GraphNode> nodesInitial,
      {Map<String, List<String>> connections}) {
    //
    //{
    //  "node-1" : ["node-6", "node-3"]
    //  "node-3" : ["node-1"]
    //}

    this.nodes = List<GraphNode>.from(nodesInitial);
    this.edges = Map<String, List<GraphEdge>>();
    this.allEdges = List<GraphEdge>();

    if (connections != null) {
      _buildEdges(connections);
    }
  }

  Graph copy() {
    Graph ret = Graph();
    ret.nodes = List<GraphNode>.from(this.nodes);
    ret.edges = Map<String, List<GraphEdge>>.from(this.edges);
    ret.allEdges = List<GraphEdge>.from(this.allEdges);
    return ret;
  }

  void _buildEdges(Map<String, List<String>> connections) {
    connections.forEach((k, v) {
      v.forEach((String id) {
        connectNodes(k, id);
      });
    });
  }

  bool nodeExists(String id) {
    return nodes.indexWhere((GraphNode n) => n.id == id) > -1;
  }

  void addNode(GraphNode n) {
    if (nodeExists(n.id)) {
      return;
    }

    nodes.add(n);
  }

  void _deleteNode(String id) {
    // remove connected edges
    if (edges.containsKey(id)) {
      for (GraphEdge e in edges[id]) {
        final int idx =
            edges[e.to].indexWhere((GraphEdge temp) => temp.to == e.from);
        if (idx < 0) continue;
        edges[e.to].removeAt(idx);

        allEdges.remove(e);
      }

      edges[id].clear();
      edges.remove(id);
    }
    nodes.removeWhere((GraphNode node) => node.id == id);
  }

  void removeNodeByID(String id) {
    _deleteNode(id);
  }

  void removeNode(GraphNode n) {
    _deleteNode(n.id);
  }

  bool checkConnection(String idA, String idB) {
    GraphEdge temp = GraphEdge(GraphEdge.INVALID_ID, idA, idB);
    if (edges[idA] == null) return false;

    final idx =
        edges[idA].indexWhere((GraphEdge e) => e.equalByEndPoints(temp));
    return idx > -1;
  }

  bool connectNodes(String idA, String idB) {
    if (checkConnection(idA, idB)) {
      // print('CONNECTION REFUSED - Nodes have been already connected.');
      return false;
    }
    String newEdgeID = GraphEdge.nextID;
    _addEdge(GraphEdge(newEdgeID, idA, idB));
    return true;
  }

  void _addEdge(GraphEdge e) {
    allEdges.add(e);

    // create, if not created yet
    edges[e.from] ??= List<GraphEdge>();
    edges[e.to] ??= List<GraphEdge>();

    GraphEdge other = GraphEdge(e.id, e.to, e.from);
    edges[e.from].add(e);
    edges[e.to].add(other);
  }

  void disconnectNodes(String from, String to) {
    GraphEdge temp = GraphEdge(GraphEdge.INVALID_ID, from, to);

    // should be one element
    edges[from]?.removeWhere((GraphEdge e) {
      return e.to == to;
    });
    edges[to]?.removeWhere((GraphEdge e) {
      return e.to == from;
    });

    allEdges.removeWhere((GraphEdge e) => e.equalByEndPoints(temp));
  }

  GraphNode getNode(String id) {
    return nodes.firstWhere((GraphNode n) => n.id == id, orElse: () => null);
  }

  List<GraphEdge> getEdges(String id) {
    return edges[id];
  }

  Map<String, List<String>> showConnections() {
    // TODO: implement
    return null;
  }

  @override
  String toString() {
    String nodesStr = nodes.fold<String>(
        '', (String prev, GraphNode n) => prev + '\n${n.toString()}');
    String edgesStr = '';
    edges.keys.forEach((String key) {
      edgesStr += '$key: to ->';
      edges[key].forEach((GraphEdge e) {
        String connectedNode = key == e.from ? e.to : e.from;

        edgesStr += ' [id:$connectedNode, biDir:${e.biDirectional}]';
      });
      edgesStr += '\n';
    });

    return nodesStr + '\n' + edgesStr;
  }
}

/*
if (edges.containsKey(id)) {
     
      for(GraphEdge e in edges[id]) {
        print('in for');
      // if (id == e.to) {
      //    print('\'FROM\' remove');
      //    edges[e.from].remove(e);
      //  }
      //  else
      //  if (id == e.from) {
          final int idx = edges[e.to].indexWhere((GraphEdge temp) => temp.to == e.from);
          if (idx < 0) continue;
          edges[e.to].removeAt(idx);

          allEdges.remove(e);
        //}
      }

      edges[id].clear();
      edges.remove(id);
    }
    nodes.removeWhere((GraphNode node) => node.id == id);
    // nodes = nodes.where((GraphNode node) => node.id != id).toList();
*/
